#!/usr/bin/env python3
# log messages and close the connection.
try: import syst3m
except: os.system("pip3 install syst3m")
import os, sys, syst3m
for i in sys.argv: print(syst3m.color.fill(i))
sys.exit(1)